package hm.wine.seller.lib;

import android.app.Application;
import android.media.AudioManager;
import android.os.Build;
import android.util.Log;
import android.webkit.JavascriptInterface;
import android.webkit.WebView;
import com.kongqw.serialportlibrary.enumerate.SerialPortEnum;
import com.kongqw.serialportlibrary.enumerate.SerialStatus;
import com.kongqw.serialportlibrary.listener.SerialPortDirectorListens;
import hm.wine.seller.MainActivity;
import hm.wine.seller.Weight.Publisher;

import java.io.File;
import java.util.Arrays;
import java.util.concurrent.atomic.AtomicBoolean;

public class JsBridge {
    private final WebView webView;
    private final String version;
    private final MainActivity.AppContext context;
    private final AudioManager audioManager;
    private final int maxVolume;

    public JsBridge(
            WebView view, String version, MainActivity.AppContext context,
            Application app, AudioManager audioManager, int maxVolume
    ) {
        this.webView = view;
        this.version = version;
        this.context = context;
        this.audioManager = audioManager;
        this.maxVolume = maxVolume;

        UartUtils.init(app, new SerialPortDirectorListens() {
            @Override
            public void onDataReceived(byte[] data, SerialPortEnum serialPortEnum) {
                if (!UartUtils.isDataCorrect(data)) return;
                switch (data[2]) {
                    case 0x11:  // weight
                        Publisher.newWeight(((data[3] & 0x00FF) << 8) + (data[4] & 0x00FF));
                        break;
                    case 0x12:  // door opened
                        if (data[3] == 0x01) jsCall("frontGateResult(true)");
                        else if (data[3] == 0x00) jsCall("backGateResult(true)");
                        else jsCall("__Error_Happened__('unknown open door result from lower machine')");
                        break;
                    case 0x13:  // pulse
                        jsCall(String.format("volumePulse(%d)", ((data[3] & 0x00FF) << 8) + (data[4] & 0x00FF)));
                        break;
                    case 0x14:  // wine out finished
                        jsCall("normalFinish()");
                        break;
                    case (byte) 0xFF:  // error
                        jsCall("__Error_Happened__('error from lower machine: 0xFF')");
                        jsCall("errorWhileOuting('error from lower machine: 0xFF')");
                        break;
                    default:
                        jsCall("__Error_Happened__('unknown data from lower machine')");
                        break;
                }
            }

            @Override
            public void onDataSent(byte[] bytes, SerialPortEnum serialPortEnum) {
                //
            }

            @Override
            public void openState(SerialPortEnum serialPortEnum, File device, SerialStatus status) {
                //
            }
        });
    }

    private void jsCall(String func_with_args) {
        String script = String.format("javascript:window.%s;", func_with_args);
        webView.post(() -> webView.evaluateJavascript(script, null));
    }

    @JavascriptInterface
    public void pin() {
        jsCall("pon()");
    }

    @JavascriptInterface
    public void onWebMounted() {
        context.disableGesture();
        jsCall(String.format("startApp('%s', '%s')", Build.SERIAL, version));
    }

    @JavascriptInterface
    public void openFrontGate() {
        UartUtils.OpenDoor(true);
    }

    @JavascriptInterface
    public void openBackGate() {
        UartUtils.OpenDoor(false);
    }

    @JavascriptInterface
    public void lightOn(int index) {
        UartUtils.SwitchLight(index, true);
    }

    @JavascriptInterface
    public void lightOff(int index) {
        UartUtils.SwitchLight(index, false);
    }

    @JavascriptInterface
    public void isCupProperlyPlaced(final int index) {
        AtomicBoolean read = new AtomicBoolean(true);
        Publisher.setSubscriber(weight -> {
            read.set(false);
            jsCall(String.format("cupProperlyPlaced(%d)", weight));
        });
        new Thread(() -> {
            try {
                while (true) {
                    Thread.sleep(300);
                    if (read.get()) UartUtils.ReadWeight(index);
                    else break;
                }
            } catch (Exception e) {
                jsCall("__Error_Happened__('application sleep error')");
            }
        }).start();
    }

    @JavascriptInterface
    public void openGas() {
        UartUtils.SwitchGas(false);
    }

    @JavascriptInterface
    public void closeGas() {
        UartUtils.SwitchGas(false);
    }

    @JavascriptInterface
    public void openValve(int index, int pulse) {
        UartUtils.OpenValve(index, pulse);
    }

    @JavascriptInterface
    public void closeValve(int index) {
        UartUtils.CloseValve(index);
    }

    @JavascriptInterface
    public void readSteadyWeight(final int index) {
        AtomicBoolean read = new AtomicBoolean(true);
        Publisher.setSubscriber(weight -> {
            read.set(false);
            jsCall(String.format("steadyWeight(%d)", weight));
        });
        new Thread(() -> {
            try {
                while (true) {
                    Thread.sleep(300);
                    if (read.get()) UartUtils.ReadWeight(index);
                    else break;
                }
            } catch (Exception e) {
                jsCall("__Error_Happened__('application sleep error')");
            }
        }).start();
    }

    @JavascriptInterface
    public void setVolume(int volume) {
        volume = Math.min(volume, maxVolume);
        if (audioManager.getStreamVolume(AudioManager.STREAM_MUSIC) > volume) {
            while (audioManager.getStreamVolume(AudioManager.STREAM_MUSIC) > volume)
                audioManager.adjustStreamVolume(AudioManager.STREAM_MUSIC, AudioManager.ADJUST_LOWER, 0);
        } else {
            while (audioManager.getStreamVolume(AudioManager.STREAM_MUSIC) < volume)
                audioManager.adjustStreamVolume(AudioManager.STREAM_MUSIC, AudioManager.ADJUST_RAISE, 0);
        }
    }

    @JavascriptInterface
    public void show() {
        context.enableGesture();
    }

    @JavascriptInterface
    public void hide() {
        context.disableGesture();
    }
}
